package org.processmining.filterd.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.filterd.gui.adapters.EventFilterModelAdapted;
import org.processmining.filterd.models.EventFilter;
import org.processmining.filterd.models.Notebook;
import org.processmining.filterd.templates.FilterTemplates;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Filter Event Log", returnLabels = { "Event Filter on Event Log" }, returnTypes = {
		EventFilterModelAdapted.class }, parameterLabels = { "Log",
				"Event Filter" }, level = PluginLevel.PeerReviewed, userAccessible = true)
public class EventFilterLoader extends NotebookLoader {

	/**
	 * Import plug-in method.
	 * 
	 * @param context
	 *            variable not used
	 * @param notebook
	 *            XML representing the notebook configuration
	 * @param log
	 *            input log which will be passed to the notebook
	 * @return serialized notebook
	 */
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D. Fahland", email = "d.fahland@tue.nl")
	@PluginVariant(variantLabel = "Filter Event Log", requiredParameterLabels = { 0 })
	public EventFilterModelAdapted load(UIPluginContext context, XLog log) {
		EventFilter notebook = FilterTemplates.TEMPLATES.getEventFilterTemplate("empty_event_filter.xpmef");
		return load(context, log, notebook);
	}

	/**
	 * Import plug-in method.
	 * 
	 * @param context
	 *            variable not used
	 * @param log
	 *            input log which will be passed to the notebook
	 * @param notebook
	 *            XML representing the notebook configuration
	 * @return serialized notebook
	 */
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Filter Event Log with Existing Filter", requiredParameterLabels = { 0, 1 })
	public EventFilterModelAdapted load(UIPluginContext context, XLog log, EventFilter notebook) {
		return new EventFilterModelAdapted(load(context, (Notebook) notebook, log));
	}

	/**
	 * Import plug-in method.
	 * 
	 * @param context
	 *            variable not used
	 * @param log
	 *            input log which will be passed to the notebook
	 * @param notebook
	 *            XML representing the notebook configuration
	 * @return serialized notebook
	 */
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Start Event Filter on Event Log", requiredParameterLabels = { 0, 1 })
	public EventFilterModelAdapted load_legacy(UIPluginContext context, XLog log, EventFilter notebook) {
		return new EventFilterModelAdapted(load(context, (Notebook) notebook, log));
	}

	public String getLabel(String label) {
		return "Event Filter on Event Log " + label;
	}
}