package org.processmining.filterd.plugins;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.filterd.configurations.NotebookConfiguration;
import org.processmining.filterd.gui.adapters.EventFilterModelAdapted;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin( //
		name = EventFilterVisualizer.NAME, //
		parameterLabels = { "Event Log", "Event Filter", "Canceller" }, //
		returnLabels = { "Event Filter on Event Log" }, //
		returnTypes = { JComponent.class }, //
		level = PluginLevel.PeerReviewed, //
		userAccessible = true, //
		//		icon = "prom_duck.png", //
		//		url = "https://www.win.tue.nl/~hverbeek/blog/2019/09/30/visualize-log-as-log-skeleton-5/", //
		help = "Start Event Filter on Event Log" //
) //
@Visualizer
public class EventFilterVisualizer extends NotebookVisualizer {

	public final static String NAME = "Event Filter on Event Log";

	@UITopiaVariant( //
			affiliation = UITopiaVariant.EHV, //
			author = "H.M.W. Verbeek", //
			email = "h.m.w.verbeek@tue.nl" //
	) //
	@PluginVariant( //
			variantLabel = "Start new Event Filter on Event Log", //
			requiredParameterLabels = { 0, 2 } //
	) //
	public JComponent run(UIPluginContext context, final XLog log, final ProMCanceller canceller) {
		return visualize(context, log, canceller, getConfig(true));
	}

	@UITopiaVariant( //
			affiliation = UITopiaVariant.EHV, //
			author = "H.M.W. Verbeek", //
			email = "h.m.w.verbeek@tue.nl" //
	) //
	@PluginVariant( //
			requiredParameterLabels = { 1, 2 } //
	) //
	public JComponent run(UIPluginContext context, EventFilterModelAdapted notebook, final ProMCanceller canceller) {
		return visualize(context, notebook, canceller, getConfig(false));
	}

	/**
	 * Returns a configuration for a simple notebook: - One computation cell - One
	 * text cell - No add-cell buttons - "Explore Event Log" as default log
	 * visualizer prefix.
	 * 
	 * @return
	 */
	public static NotebookConfiguration getConfig(boolean createCells) {
		NotebookConfiguration config = new NotebookConfiguration();
		/*
		 * Use JavaFX panels without the options to add cells.
		 */
		config.setNotebookFileName("EventFilter/Notebook.fxml");
		config.setComputationCellFileName("EventFilter/ComputationCell.fxml");
		config.setTextCellFileName("EventFilter/TextCell.fxml");
		/*
		 * Use the "Explore Event Log"visualizer by default.
		 */
		config.setPreferredVisualizerPrefix("Explore Event Log");
		if (createCells) {
			/*
			 * Automatically create a computation cell titled "Filter Stack" and a text cell
			 * titled "Description".
			 */
			config.getCells().add("CFilter Stack");
			config.getCells().add("TDescription");
		}
		/*
		 * Set auto size to true. This causes the computation cell to be resized in such
		 * a way that the computation cell and text cell fit the screen.
		 */
		config.setAutoSize(true);

		config.setHideEventLog(true);

		config.setNotebookLabel("Event Filter");
		/*
		 * Return this configuration.
		 */
		return config;
	}
}
